#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import clyngor
import csv
import os
import sys
import time

# Path to package scripts.
global root
root = os.path.dirname(__file__)

def check_folder(folder_in):
    '''
    Args:
        folder_in (str): folder to check
    '''
    if not os.path.isdir(folder_in):
        try:
            os.makedirs(folder_in)
        except OSError:
            raise OSError('Can not create folder: ' + folder_in)


def mz_computation(input_file):
    '''
    Compute MZ for all known molecules and MZ for reaction.
    Return the result as a string.
    Use next because for these analysis, we expect only one answer.

    Args:
        input_file (str): path to the input data file
    Returns:
        mz_result (str): ASP answer as str
    '''
    print('~~~~~Creation of MZ~~~~~')
    mzcomputation_path = os.path.join(*[root, 'asp', 'MZComputation.lp'])
    mz_solver = clyngor.solve([input_file, mzcomputation_path], use_clingo_module=False)
    mz_result = '\n'.join([atom+'.' for atom in next(mz_solver.parse_args.atoms_as_string.int_not_parsed.sorted)])

    return mz_result


def reaction_creation(input_file, output_folder):
    '''
    Detect reaction sites by comparing molecules implied in a reaction.
    Return the result as a string.

    Args:
        input_file (str): path to the input data file
        output_folder (str): path to the output folder
    Returns:
        reaction_result (str): ASP answer as str
    '''
    print('~~~~~Creation of Reaction~~~~~')
    reaction_site_extraction_script = os.path.join(*[root, 'asp', 'ReactionSiteExtraction.lp'])
    reaction_solver = clyngor.solve([input_file, reaction_site_extraction_script], use_clingo_module=False)
    reaction_results = []
    transformation_reactants = {}
    transformation_products = {}
    reactions = []
    for atom in next(reaction_solver.parse_args.int_not_parsed.sorted):
        reaction_results.append(atom[0] + '(' + ','.join(atom[1]) + ')')
        if 'diff' in atom[0]:
            reaction_id = atom[1][0]
            substructures = atom[1][1:]
            reactions.append(reaction_id)
            if 'Before' in atom[0]:
                if reaction_id not in transformation_reactants:
                    transformation_reactants[reaction_id] = [substructures]
                else:
                    transformation_reactants[reaction_id].append([substructures])
            elif 'After' in atom[0]:
                if reaction_id not in transformation_products:
                    transformation_products[reaction_id] = [substructures]
                else:
                    transformation_products[reaction_id].append([substructures])

    reactions = set(reactions)
    pathmodel_output_transformation_path = os.path.join(output_folder, 'pathmodel_data_transformations.tsv')
    with open(pathmodel_output_transformation_path, 'w') as transformation_file:
        csvwriter = csv.writer(transformation_file, delimiter = '\t')
        csvwriter.writerow(['reaction_id', 'reactant_sbustructure', 'product_substructure'])
        for reaction in reactions:
            if reaction in transformation_reactants:
                reactant = transformation_reactants[reaction]
            else:
                reactant = []
            if reaction in transformation_products:
                product = transformation_products[reaction]
            else:
                product = []
            csvwriter.writerow([reaction, reactant, product])


    reaction_result = '\n'.join([atom+'.' for atom in reaction_results])

    return reaction_result


def extract_transformation_known_reactions(input_file, output_folder):
    print('~~~~~Extraction of Reaction~~~~~')
    reaction_extraction_script = os.path.join(*[root, 'asp', 'CompareMolecules.lp'])
    reaction_solver = clyngor.solve([input_file, reaction_extraction_script], use_clingo_module=False)

    known_transformations_path = os.path.join(output_folder, 'known_transformations.tsv')
    with open(known_transformations_path, 'w') as output_file:
        csvwriter = csv.writer(output_file, delimiter='\t')
        csvwriter.writerow(['reaction_name', 'molecule_A', 'molecule_B', 'bond_type', 'atom_1', 'atom_2'])
        for atom in next(reaction_solver.parse_args.int_not_parsed.sorted):
            if 'diff' in atom[0]:
                reaction_id = atom[1][0]
                molecule_A_name = atom[1][1]
                molecule_B_name = atom[1][2]
                bond_type = atom[1][3]
                atom_1 = atom[1][4]
                atom_2 = atom[1][5]
                csvwriter.writerow([reaction_id, molecule_A_name, molecule_B_name, bond_type, atom_1, atom_2])


def pathmodel_inference(input_string, output_folder, step_limit):
    '''
    Infer reactions and metabolites from known reactions and metabolites.

    Args:
        input_file (str): path to the input data file
        output_folder (str): path to the output folder
        step_limit (int): if PathModel reaches this step limit, it will stop its inference and returns an error
    Returns:
        pathmodel_result (str): ASP answer as str
    '''
    print('~~~~~Inference of reactions and metabolites~~~~~')
    pathmodel_script_path = os.path.join(*[root, 'asp', 'PathModel.lp'])
    pathmodel_solver = clyngor.solve(inline=input_string, files=pathmodel_script_path, use_clingo_module=False)

    # Take the best model.
    best_model = None
    best_model = []
    pathways = {}
    reactions = {}

    for atom in next(pathmodel_solver.parse_args.int_not_parsed.sorted):
        if 'inferred' in atom[0]:
            infer_step = int(atom[1][1])
            reactant = atom[1][0][1][0]
            product = atom[1][0][1][1]
            if infer_step not in pathways:
                pathways[infer_step] = [(reactant, product)]
            else:
                pathways[infer_step].append((reactant, product))
        elif 'query' in atom[0]:
            step = int(atom[1][0])
            if step_limit:
                if int(step_limit) == step:
                    sys.exit('Step limit ('+str(step_limit)+') reached, the goal is not reachable in ' + str(step_limit) + 'steps or there is an error in the data.')
        else:
            best_model.append(atom[0] + '(' + ','.join(atom[1]) + ')')
            if 'newreaction' in atom[0]:
                reactions[atom[1][1:]] = atom[1][0]

    pathmodel_result = '\n'.join([atom+'.' for atom in best_model])

    already_inferreds = []
    pathmodel_incremental_inference_path = os.path.join(output_folder, 'pathmodel_incremental_inference.tsv')
    with open(pathmodel_incremental_inference_path, 'w') as outfile:
        csvwriter = csv.writer(outfile, delimiter='\t')
        csvwriter.writerow(["infer_step", "new_reaction", "reactant", "product"])
        for infer_step in sorted(list(pathways.keys())):
            for reaction in pathways[infer_step]:
                if reaction in reactions and reaction not in already_inferreds:
                    csvwriter.writerow([infer_step, reactions[reaction], *reaction])
                    already_inferreds.append(reaction)

    return pathmodel_result


def pathmodel_analysis(input_file, output_folder, step_limit=None):
    '''
    Run all PathModel functions.

    Args:
        input_file (str): path to the input data file
        output_folder (str): path to the output folder
        step_limit (int): if PathModel reaches this step limit, it will stop its inference and returns an error
    Returns:
        pathmodel_result (str): ASP answer as str
    '''
    check_folder(output_folder)

    mz_result = mz_computation(input_file)

    extract_transformation_known_reactions(input_file, output_folder)

    reaction_result = reaction_creation(input_file, output_folder)

    # Create step limit.
    if step_limit:
        str_step_limit = "step_limit("+str(step_limit)+")."
    else:
        step_limit = 100
        str_step_limit = "step_limit(100)."

    # Merge input files + result from MZ prediction and reaction creation into a string, which will be the input file for PathModel.
    input_string = open(input_file, 'r').read() + '\n' + str_step_limit + '\n' + mz_result + '\n' + reaction_result

    data_pathmodel_output_path = os.path.join(output_folder, 'data_pathmodel.lp')
    with open(data_pathmodel_output_path, 'w') as intermediate_file:
        intermediate_file.write(input_string)
        intermediate_file.write('\n')

    pathmodel_result = pathmodel_inference(input_string, output_folder, step_limit)

    output_lp = os.path.join(output_folder, 'pathmodel_output.lp')

    print('~~~~~Creating result file~~~~~')
    # Write input in a file.
    with open(output_lp, "w") as resultfile:
        resultfile.write(pathmodel_result)
        resultfile.write('\n')

    return pathmodel_result



import argparse
import os
import subprocess
import sys
import time

from pathmodel.pathmodel_wrapper import check_folder, pathmodel_analysis

def main():
    '''
    Arguments when used with entrypoint as: pathmodel -d data.lp
    '''
    parser = argparse.ArgumentParser(
        "pathmodel",
        description="Prototype of the metabolic pathway drift hypothesis. For specific help on each subcommand use: pathmodel {cmd} --help",
    )
    # parent parser
    parent_parser_i = argparse.ArgumentParser(add_help=False)
    parent_parser_i.add_argument(
        "-i",
        "--input",
        metavar="INPUT_FILE",
        help="input file containing atoms, bonds, reactions and goal",
        required=True)
    parent_parser_o = argparse.ArgumentParser(add_help=False)
    parent_parser_o.add_argument(
        "-o",
        "--out",
        dest="output_folder",
        required=True,
        help="output directory path",
        metavar="OUPUT_DIR")
    parent_parser_s = argparse.ArgumentParser(add_help=False)
    parent_parser_s.add_argument(
        "-s",
        "--step-limit",
        dest="step_limit",
        required=False,
        help="set a maximal number of step to avoid endless run of pathmodel (by default at 100)",
        metavar="INT")

    # subparsers
    subparsers = parser.add_subparsers(
        title='subcommands',
        description='valid subcommands:',
        dest="cmd")
    infer_parser = subparsers.add_parser(
        "infer",
        help="PathModel inference on data",
        parents=[
            parent_parser_i, parent_parser_o, parent_parser_s
        ],
        description=
        "Run PathModel on input data lp file"
        )
    test_parser = subparsers.add_parser(
        "test",
        help="test PathModel on data from the article",
        parents=[
            parent_parser_o
        ],
        description=
        "Test PathModel on the data from the article (sterol and MAA)"
        )

    parser_args = parser.parse_args()

    # Print help and exit if no arguments.
    argument_number = len(sys.argv[1:])
    if argument_number == 0:
        parser.print_help()
        parser.exit()

    output_folder = parser_args.output_folder

    if parser_args.cmd == 'test':
        package_path = os.path.dirname(os.path.realpath(__file__))
        data_package_path = os.path.join(package_path, 'data')
        sterol_input_path = os.path.join(data_package_path, 'sterol_pwy.lp')
        maa_input_path = os.path.join(data_package_path, 'MAA_pwy.lp')
        sterol_out = os.path.join(output_folder, 'sterol')
        maa_out = os.path.join(output_folder, 'MAA')

        check_folder(sterol_out)
        check_folder(maa_out)

        pathmodel_analysis(sterol_input_path, sterol_out, None)
        pathmodel_analysis(maa_input_path, maa_out, None)

        return

    elif parser_args.cmd == 'infer':
        input_file = parser_args.input
        step_limit = parser_args.step_limit
        pathmodel_analysis(input_file, output_folder, step_limit)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import shutil
import pathmodel

def test_MZcomputation():
	result = pathmodel.mz_computation('pathmodel_test_data.lp')

	result_expected = """domain("molecule_1",triangle).
domain("molecule_2",triangle).
domain("molecule_3",triangle).
domain("molecule_4",triangle).
domain("molecule_5",triangle).
moleculeComposition("molecule_1",4,8,0,0,0).
moleculeComposition("molecule_2",4,6,0,0,0).
moleculeComposition("molecule_3",6,10,0,0,0).
moleculeComposition("molecule_4",6,8,0,0,0).
moleculeComposition("molecule_5",7,10,0,0,0).
moleculeMZ("molecule_1",561020).
moleculeMZ("molecule_2",540872).
moleculeMZ("molecule_3",821382).
moleculeMZ("molecule_4",801234).
moleculeMZ("molecule_5",941489).
moleculeNbAtoms("molecule_1",12).
moleculeNbAtoms("molecule_1",4).
moleculeNbAtoms("molecule_2",10).
moleculeNbAtoms("molecule_2",4).
moleculeNbAtoms("molecule_3",16).
moleculeNbAtoms("molecule_3",6).
moleculeNbAtoms("molecule_4",14).
moleculeNbAtoms("molecule_4",6).
moleculeNbAtoms("molecule_5",17).
moleculeNbAtoms("molecule_5",7).
numberTotalBonds("molecule_1",4).
numberTotalBonds("molecule_2",4).
numberTotalBonds("molecule_3",7).
numberTotalBonds("molecule_4",7).
numberTotalBonds("molecule_5",8).
reactionMZ(reduction,20148)."""

	assert result == result_expected

def test_transformation():
	pathmodel.check_folder('test_output')
	result = pathmodel.reaction_creation('pathmodel_test_data.lp', 'test_output')

	result_expected = """diffBondAfterReaction(reduction,double,2,4).
diffBondBeforeReaction(reduction,single,2,4).
siteAfterReaction(reduction,"molecule_2").
siteAfterReaction(reduction,"molecule_4").
siteBeforeReaction(reduction,"molecule_1").
siteBeforeReaction(reduction,"molecule_3").
siteBeforeReaction(reduction,"molecule_5")."""
	shutil.rmtree('test_output')
	assert result == result_expected

def test_pathmodel():
	resultfile_content = pathmodel.pathmodel_analysis('pathmodel_test_data.lp', 'test_output')

	result_expected = """newreaction(reduction,"molecule_3","molecule_4").
newreaction(reduction,"molecule_5","Prediction_921341_reduction").
predictatom("Prediction_921341_reduction",1,carb).
predictatom("Prediction_921341_reduction",2,carb).
predictatom("Prediction_921341_reduction",3,carb).
predictatom("Prediction_921341_reduction",4,carb).
predictatom("Prediction_921341_reduction",5,carb).
predictatom("Prediction_921341_reduction",6,carb).
predictatom("Prediction_921341_reduction",7,carb).
predictbond("Prediction_921341_reduction",double,2,4).
predictbond("Prediction_921341_reduction",double,3,6).
predictbond("Prediction_921341_reduction",single,1,2).
predictbond("Prediction_921341_reduction",single,1,3).
predictbond("Prediction_921341_reduction",single,1,6).
predictbond("Prediction_921341_reduction",single,1,7).
predictbond("Prediction_921341_reduction",single,2,3).
predictbond("Prediction_921341_reduction",single,5,6).
reaction(reduction,"molecule_1","molecule_2").
reaction(reduction,"molecule_3","molecule_4").
reaction(reduction,"molecule_5","Prediction_921341_reduction")."""
	shutil.rmtree('test_output')
	assert resultfile_content == result_expected


import os
from distutils.util import convert_path
from setuptools import setup, find_packages

init_data = {}
init_pathname = convert_path('pathmodel/__init__.py')
with open(init_pathname) as init_file:
    exec(init_file.read(), init_data)

setup_directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(setup_directory, 'README.rst'), encoding='utf-8') as readme_file:
    long_description = readme_file.read()

setup(
    name='pathmodel',
    version=init_data['__version__'],
    url='https://github.com/pathmodel/pathmodel',
    description='Ab initio pathway inference',
    long_description=long_description,
    author='A. Belcour',
    packages=['pathmodel'],
    package_dir={'pathmodel': 'pathmodel'},
    package_data = {'pathmodel' : ['asp/*.lp', 'data/*.lp']},
    entry_points={
        'console_scripts': [
            'pathmodel = pathmodel.__main__:main',
            'pathmodel_plot = pathmodel.plotting:run_pathway_creation'
        ]
    },
    install_requires=['clyngor', 'matplotlib', 'clyngor-with-clingo', 'networkx'],
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

"""
Script to plot 2D representation of molecule present in an ASP input file.

This is useful to check if the ASP structure corresponds to the molecule structure.
"""

import argparse
import os
import networkx as nx
# import for using the script in docker.
import matplotlib; matplotlib.use('svg')
import matplotlib.pyplot as plt

from clyngor import ASP
from pathmodel import check_folder
from networkx.drawing.nx_agraph import graphviz_layout

try:
    from rdkit import Chem
    from rdkit.Chem import AllChem
    from rdkit.Chem import Draw
except ImportError:
    raise ImportError("Requires rdkit (https://github.com/rdkit/rdkit).")

try:
    import pygraphviz
except ImportError:
    raise ImportError("Requires graphviz (https://www.graphviz.org/) and pygraphviz (https://pygraphviz.github.io/).")


def run_pathway_creation():
    """
    Functions handling the argument parsing of pathmodel_plot.
    """
    parser = argparse.ArgumentParser(description="Plot molecules and reactions inferred by PathModel.")
    parser.add_argument("-i", "--input", dest="input_folder", metavar="FOLDER", help="Input folder corresponds to the output folder of pathmodel.")

    parser_args = parser.parse_args()

    input_folder = parser_args.input_folder
    pathmodel_output_file = os.path.join(input_folder, 'pathmodel_output.lp')
    with open(pathmodel_output_file, 'r') as pathmodel_output:
        asp_code = pathmodel_output.read()
    picture_name = os.path.join(input_folder, 'pathmodel_output.svg')
    input_filename = os.path.join(input_folder, 'data_pathmodel.lp')
    output_repository = os.path.join(input_folder, 'molecules')
    new_output_repository = os.path.join(input_folder, 'newmolecules_from_mz')

    # Check if output folder exists if not create it.
    check_folder(output_repository)
    check_folder(new_output_repository)

    print('~~~~~Creating result picture~~~~~')
    pathmodel_pathway_picture(asp_code, picture_name, input_filename)

    if 'predictatom' in asp_code and 'predictbond' in asp_code:
        create_2dmolecule(pathmodel_output_file, new_output_repository, align_domain=None)

    print('~~~~~Creating molecules pictures~~~~~')
    create_2dmolecule(input_filename, output_repository, False)


def pathmodel_pathway_picture(asp_code, picture_name, input_filename):
    """
    Create the pathway picture using ASP results code from PathModel inference.

    Args:
        asp_code (str): string containing PathModel results
        picture_name (str): path to the output picture file
        input_filename (str): path to PathModel intermediary file
    """
    DG = nx.DiGraph()

    known_compounds = []
    inferred_compounds = []

    known_reactions = []
    inferred_reactions = []

    absent_molecules = []

    with open(input_filename, 'r') as intermediate_file:
        for answer in ASP(intermediate_file.read(), use_clingo_module=False).parse_args.by_predicate.discard_quotes:
            for predicate in answer:
                if predicate == "absentmolecules":
                    for atom in answer[predicate]:
                        absent_molecules.append(atom[0])

    for answer in ASP(asp_code, use_clingo_module=False).parse_args.by_predicate.discard_quotes:
        for predicate in answer:
            for atom in answer[predicate]:
                reaction = atom[0]
                reactant = atom[1]
                product = atom[2]
                if predicate == "reaction":
                    if reactant not in absent_molecules:
                        known_compounds.append(reactant)
                    if product not in absent_molecules:
                        known_compounds.append(product)

                    if product not in absent_molecules and reactant not in absent_molecules:
                        known_reactions.append((reactant, product))
                        DG.add_edge(reactant, product, label=reaction)
                elif predicate == "newreaction":
                    if 'Prediction_' in reactant:
                        inferred_compounds.append(reactant)
                    if 'Prediction_' in product:
                        inferred_compounds.append(product)

                    inferred_reactions.append((reactant, product))
                    DG.add_edge(reactant, product, label=reaction)
    plt.figure(figsize=(25, 25))

    nx.draw_networkx_nodes(DG,
                           graphviz_layout(DG, prog='neato'),
                           nodelist=known_compounds,
                           node_color="green",
                           node_size=3000,
                           node_shape='s',
                           alpha=0.5)
    nx.draw_networkx_nodes(DG,
                           graphviz_layout(DG, prog='neato'),
                           nodelist=inferred_compounds,
                           node_color="blue",
                           node_size=2000,
                           node_shape='s',
                           alpha=0.5)

    nx.draw_networkx_edges(DG,
                           graphviz_layout(DG, prog='neato'),
                           edgelist=known_reactions,
                           edge_color="green",
                           alpha=0.5,
                           width=2.0,
                           arrow=True,
                           arrowstyle='->',
                           arrowsize=14)
    nx.draw_networkx_edges(DG,
                           graphviz_layout(DG, prog='neato'),
                           edgelist=inferred_reactions,
                           edge_color="blue",
                           alpha=0.5,
                           width=2.0,
                           arrow=True,
                           arrowstyle='->',
                           arrowsize=14)
    nx.draw_networkx_labels(DG,
                            graphviz_layout(DG, prog='neato'),
                            font_size=15)

    ax = plt.gca()
    ax.set_axis_off()

    extension = os.path.splitext(picture_name)[1].strip('.')
    plt.savefig(picture_name, dpi=144, format=extension)


def create_rdkit_molecule(molecule_name, molecules, molecule_numberings, bonds):
    '''
    Using dictionaries containing molecule structure create a rdkit molecule.

    Args:
        molecule_name (str): name of a molecule
        molecules (dict): dictionary containing for each molecules a list with a atom numbers and types in the molecule
        molecule_numberings (dict): dictionary containing for each molecules a list with a atom numbers in the molecule
        bonds (dict): dictionary containing for each molecules the list of its bond in utpels (bond_number_1, bond_number_2, bond_type)
    Returns:
        rdmol (Mol): the molecule in rdkit Molecule
    '''
    # Create an editable molecule.
    rdmol = Chem.Mol()
    rdedmol = Chem.EditableMol(rdmol)

    atoms = {atom_tuple[0]: atom_tuple[1] for atom_tuple in sorted(molecules[molecule_name])}
    atom_numberings = sorted(molecule_numberings[molecule_name])
    # Renumber atom so there is no atom with a number superior to the number of atoms in the molecule.
    atom_replaces = {}

    for atom in atom_numberings:
        if atom != sorted(atom_numberings).index(atom)+1:
            atom_replaces[atom] = sorted(atom_numberings).index(atom)+1

    # Add a first atom to keep most of the atom numbering.
    rdatom = Chem.Atom(0)
    rdedmol.AddAtom(rdatom)

    # Add atoms from the molecule.
    # Add absent atoms to keep the atom numbering.
    for atom_number in range(max(atom_numberings)):
        atom_number += 1
        if atom_number in atoms:
            atom = atoms[atom_number]
        else:
            atom = 0
        rdatom = Chem.Atom(atom)
        rdedmol.AddAtom(rdatom)

    # Add bonds from the molecule.
    for bond in bonds[molecule_name]:
        bond = tuple(bond)
        rdedmol.AddBond(bond[0],
                        bond[1],
                        bond[2])

    # Create molecule.
    rdmol = rdedmol.GetMol()

    return rdmol


def create_2dmolecule(input_filename, output_directory, align_domain=None):
    '''
    From an ASP input file create 2d representation of molecules.
    To use align_domain, you need the intermediate file creates by pathmodel_wrapper.py.
    With align_domain, rdkit will use domain to align molecules.

    Args:
        input_filename (str): path to PathMoldel output file
        output_directory (str): output folder containing pictures of the molecuels and of the infered pathway
        align_domain (bool): if True, rdkit will use domain to align molecules
    '''
    with open(input_filename, 'r') as input_file:
        asp_code = input_file.read()
    # Set bond types transformation from ASP to rdkit.
    bondtypes = {'single': Chem.BondType.SINGLE,
                    'singleS': Chem.BondType.SINGLE,
                    'singleR': Chem.BondType.SINGLE,
                    'double': Chem.BondType.DOUBLE,
                    'triple': Chem.BondType.TRIPLE,
                    'variable': Chem.BondType.UNSPECIFIED}

    # Set atomic number transformation from ASP to rdkit.
    atomicNumber = {'carb': 6,
                    'nitr': 7,
                    'oxyg': 8,
                    'phos': 15,
                    'variable': 0}

    if align_domain:
        domain_molecules = {}
        domain_molecule_numberings = {}
        domain_bonds = {}
        molecule_domains = {}

    molecules = {}
    molecule_numberings = {}
    bonds = {}

    # Parse ASP input file and extract molecules, atoms and bonds.
    for predicate in ASP(asp_code, use_clingo_module=False).parse_args.discard_quotes:
        for variable in predicate:
            if variable[0] == 'atom' or variable[0] == 'predictatom':
                atom_molecule = variable[1][0]
                atom_number = variable[1][1]
                atom_type = atomicNumber[variable[1][2]]
                if atom_molecule not in molecules:
                    molecules[atom_molecule] = [(atom_number, atom_type)]
                    molecule_numberings[atom_molecule] = [atom_number]
                else:
                    molecules[atom_molecule].append((atom_number, atom_type))
                    molecule_numberings[atom_molecule].append(atom_number)

            elif variable[0] == 'bond' or variable[0] == 'predictbond':
                atom_molecule = variable[1][0]
                bond_number_1 = variable[1][2]
                bond_number_2 = variable[1][3]
                bond_type = bondtypes[variable[1][1]]
                if atom_molecule not in bonds:
                    bonds[atom_molecule] = [(bond_number_1, bond_number_2, bond_type)]
                else:
                    bonds[atom_molecule].append((bond_number_1, bond_number_2, bond_type))

            if align_domain:
                # Extract domain information.
                if variable[0] == 'atomDomain':
                    atom_molecule = variable[1][0]
                    atom_number = variable[1][1]
                    atom_type = atomicNumber[variable[1][2]]
                    if atom_molecule not in domain_molecules:
                        domain_molecules[atom_molecule] = [(atom_number, atom_type)]
                        domain_molecule_numberings[atom_molecule] = [atom_number]
                    else:
                        domain_molecules[atom_molecule].append((atom_number, atom_type))
                        domain_molecule_numberings[atom_molecule].append(atom_number)

                elif variable[0] == 'bondDomain':
                    atom_molecule = variable[1][0]
                    bond_number_1 = variable[1][2]
                    bond_number_2 = variable[1][3]
                    bond_type = bondtypes[variable[1][1]]
                    if atom_molecule not in domain_bonds:
                        domain_bonds[atom_molecule] = [(bond_number_1, bond_number_2, bond_type)]
                    else:
                        domain_bonds[atom_molecule].append((bond_number_1, bond_number_2, bond_type))

                elif variable[0] == 'domain':
                    molecule_name = variable[1][0]
                    domain_name = variable[1][1]
                    molecule_domains[molecule_name] = domain_name

    # For each domains, create the corresponding rdkit molecule.
    if align_domain:
        rddomains = {}
        for domain_name in domain_molecules:
            rddomain = create_rdkit_molecule(domain_name, domain_molecules, domain_molecule_numberings, domain_bonds)
            rddomains[domain_name] = rddomain

    # For each molecules, create a rdkit molecule.
    for molecule_name in molecules:
        rdmol = create_rdkit_molecule(molecule_name, molecules, molecule_numberings, bonds)

        if align_domain:
            # Use domain to align molecule.
            template = rddomains[molecule_domains[molecule_name]]
            AllChem.Compute2DCoords(rdmol)
            AllChem.Compute2DCoords(template)
            AllChem.GenerateDepictionMatching2DStructure(rdmol, template)

        # Draw molecule.
        molecule_name = molecule_name
        print(molecule_name)
        output_molecule_path = os.path.join(output_directory, molecule_name+'.svg')
        Draw.MolToFile(rdmol, output_molecule_path, size=(800, 800), includeAtomNumbers=True)

    input_file.close()


if __name__ == '__main__':
    run_pathway_creation()
